/* 
 * Help facility.
 *
 * If the HELPFILE string is #defined, then the help facility will be 
 * available in one way or another.
 *
 * If the environment variable PAGER is set, that pager will be used to
 * display the help file.  Otherwise, if a PAGER command was #defined
 * at compile-time then that command will be used.  If neither PAGER is
 * set, then the built-in simple-minded pager will be used instead.
 *
 * For machines tight on memory space:
 * Leaving HELPFILE undefined conserves the most memory (but makes the
 * help facility unavailable).  Defining HELPFILE but leaving the (macro) 
 * PAGER undefined consumes the most memory since the simple-minded pager 
 * will be compiled into the editor.  If you intend to use the environment
 * variable PAGER for help, you should also #define PAGER to be something
 * as well.
 */
#define HELPFILE 	"/usr/local/lib/jstevie/stevie.hlp" /* */
#define PAGER 		"less" /* */


#include "stevie.h"

extern int fexpand();
#define PRIVATE static
PRIVATE void dumb_pager();

/*
 *  Display help file using a pager.  If it can't determine a pager
 *  to use, then it will use a built-in simple-minded pager.
 */
PUBLIC
void ex_help()
{
#ifdef HELPFILE
    register char   *pager, *file, *p;
    char            cmd[MAXCLIN];
    extern char     *getenv();

    /*
     * Get help filename and pager command.
     */
    file = HELPFILE;
    if ((file == NULL) || (*file == NUL)) {
        emsg("Help file name not set");
        return;
    } 

    if ((pager = getenv("PAGER")) == NULL) {
#ifdef PAGER
        pager = PAGER;
#else
		dumb_pager(file);	/* use built-in pager */
		return;
#endif
    }   

    /*
     * Check if pager command already has a '%s' in it.
     * If command does have a '%s', the helpfile will be substituted there.
     * Otherwise, the helpfile will come after the pager command.
     *
     * Examples: "nroff -ms %s | more"  ---> "nroff -ms helpfile | more"
     *           "less"                 ---> "less helpfile"
     */
    for (p = pager;  (*p && (*p != '%'));  p++);
    if ((p[0] == '%') && (p[1] == 's')) {   /* found a `%s' */
        sprintf(cmd, pager, file);
    } else {
        sprintf(cmd, "%s %s", pager, file);
    }

#if 0
    smsg("\"%s\"", cmd);        /* debugging */
    wait_return();
#endif

    /*
     * Expand wildcards and execute it.
     */
	toutstr(T_ED, 0);
    fexpand(cmd, MAXCLIN);
    doshell(cmd);


#else   /* HELPFILE isn't defined */

    emsg("Help file not configured");

#endif
}


#ifdef HELPFILE
#ifndef PAGER
/*
 *  Prompt for user input.  Returns TRUE if user is quitting,
 *  FALSE if user wants to continue.
 */
PRIVATE
bool_t prompt()
{
    static char msg1[] = "<Press space to continue>";
    static char msg2[] = "<Any other key will quit>";

    windgoto(Rows-2, (Columns - (sizeof(msg1) + 1)));
        outstr(msg1);

    windgoto(Rows-1, (Columns - (sizeof(msg2) + 1)));
        outstr(msg2);

    return(vgetc() != ' ');     /* abort if not a space */
}


/*
 *  This implements a simple pager.  It displays each line from
 *  a file on the next screen row.  It prints a prompt when it reaches
 *  the bottom of the screen.
 */
PRIVATE
void dumb_pager(name)
    char *name;
{
    register FILE   *fp;
    register int    row;
    uchar           line[MAXMLEN];
    bool_t          quit;

    /*
     * Open the file.
     */
    if ((fp = fopen(name, "r")) == NULL) {
        emsg("Can't open help file"); /* emsg("Can't open help file \"%s\"", name); */
        return;
    }

    /*
     * Load each line from the file and display it on the next screen row.
     * When we reach the bottom of the screen print a message asking if
     * we should continue.
     */
    quit = FALSE;
    screenclear();  
    for (row = 0;  (fgets(line, MAXMLEN, fp) != NULL);  row++) {
        windgoto(row, 0);
        outstr(line);

        if (row == Rows-2) {    /* reached bottom of screen; prompt user */
            if (quit = prompt())    /* (note `=' here) */
                break;
            screenclear();  
            row = -1;
        }
    }
    if (!quit)
        prompt();

    /* 
     * Close the file and return back to the edit screen 
     */
    fclose(fp);
    screenclear();
    updatescreen();
}

#endif  /* !defined(PAGER) */
#endif  /* defined(HELPFILE) */
